export type ApiResponse = Readonly<
    {
        id: number;
        slug: string;
        parent: number | null;
    }[]
>;

export type Sitemap = Readonly<{
    readonly name: string;
    readonly id: number;
    readonly children: readonly Sitemap[];
}>;

export type Simplify<T> = T extends Record<string, unknown> | unknown[]
  ? T extends infer O
    ? { [K in keyof O]: Simplify<O[K]> }
    : never
  : T;

type PathsForEntry<TEntry extends Sitemap, TPrefix extends string> = TEntry extends unknown ? [`${TPrefix}/${TEntry['name']}`, TEntry['id']] | PathsForEntry<TEntry['children'][number], `${TPrefix}/${TEntry['name']}`> : never;

export type Paths<TSitemap extends Sitemap> = Simplify<PathsForEntry<TSitemap['children'][number], ''> | ['/', TSitemap['id']]>;

type Mew<T> = T extends readonly [string, number] ? T : never;

type ResetStack<T> = T extends infer R ? R & readonly [string, number] : never;

export type ResolverForPaths<TPaths extends readonly [string, number]> = {
    <TPath extends TPaths[0]>(path: TPath): Extract<TPaths, readonly [TPath, number]>[1];
    (unknownPath: string): null;
};

export type PathsResolver<TSitemap extends Sitemap> = ResolverForPaths<Extract<Paths<TSitemap> extends infer R ? R : never, readonly [string, number]>>;

//export type PathsResolver<TSitemap extends Sitemap> = ResolverForPaths<Paths<TSitemap>>;
